# header: importing useful modules and functions

import csv
import numpy as np
import matplotlib.pyplot as plt
from scipy.optimize import fmin


#%%#########################################################
# PART 1: Modeling the global carbon cycle
############################################################

#============
# SECTION 2
#============


# definition of the impulse response function for atmospheric CO2
def irf_CO2(t):
    irf =  a_C0 + a_C1 * np.exp(-t/tau_C1) + a_C2 * np.exp(-t/tau_C2) + a_C3 * np.exp(-t/tau_C3)
    return irf


# with the following values of the parameters
a_C0 = 0.2173
a_C1 = 0.2240
a_C2 = 0.2824
a_C3 = 0.2763
tau_C1 = 394.4 #{yr}
tau_C2 = 36.54 #{yr}
tau_C3 = 4.304 #{yr}

### Figure 2.1 ###
# plot the IRF for atmospheric CO2
plt.figure()
# over the first 100 years
time1 = np.arange(0,100)
plt.subplot(1,2,1)
plt.plot(time1,irf_CO2(time1))
plt.plot(time1,a_C0*np.ones(np.shape(time1)),ls='--')
# up to 1000 years
time2 = np.arange(100,1000)
plt.subplot(1,2,2)
plt.plot(time2,irf_CO2(time2),label='irf_CO2')
plt.plot(time2,a_C0*np.ones(np.shape(time2)),ls='--',label='a_C0')
# polishing the figure
plt.subplot(1,2,1)
plt.axis([0,100,0,1])
plt.xlabel('time (yr)')
plt.ylabel('airborne fraction (.)')
plt.subplot(1,2,2)
plt.axis([100,1000,0,1])
plt.xticks([200,400,600,800,1000])
plt.yticks([0,0.2,0.4,0.6,0.8,1],['','','','','',''])
plt.legend(loc=0)
plt.subplots_adjust(left=0.10,bottom=0.10,right=0.95,top=0.95,wspace=0.01)


#- Question 2
# Individual behaviour of IRF terms
plt.figure()
# over the first 100 years
time1 = np.arange(0,1000)
plt.subplot(2,2,1)
plt.plot(time1, a_C0+time1*0 )
plt.subplot(2,2,2)
plt.plot(time1, a_C1 * np.exp(-time1/tau_C1) ) # Ocean
plt.subplot(2,2,3)
plt.plot(time1, a_C2 * np.exp(-time1/tau_C2) ) # Land
plt.subplot(2,2,4)
plt.plot(time1, a_C3 * np.exp(-time1/tau_C3) ) # Atmosphere 


#- Question 5
# definition of the carbon sink function
def irf_Csinks(t):
    irf =  -a_C1/tau_C1 * np.exp(-t/tau_C1) - a_C2/tau_C2 * np.exp(-t/tau_C2) - a_C3/tau_C3 * np.exp(-t/tau_C3)
    return irf

### Fig. 2.2 ###
# plot the C sinks response to a pulse of emissions
plt.figure()
time = np.arange(100)
plt.plot(time,irf_Csinks(time))
plt.ylabel('C sinks response (/yr)')
plt.xlabel('time (yr)')



#%%==========
# SECTION 3
#============

# load the historical data on fossil CO2 emissions
EFF_type = np.array([line for line in csv.reader(open('data/CDIAC_EFF.global.1751_2011.csv','r'))][2:],dtype=float)[:,2:-1] / 1000 * 44/12. #{GtCO2/yr}
EFF_lgd = [line for line in csv.reader(open('data/CDIAC_EFF.global.1751_2011.csv','r'))][0][2:-1]
EFF = np.sum(EFF_type,1)

### Fig. 3.1 ###
# plot the different emissions
plt.figure()
for n in range(5):
    plt.plot(np.arange(1751,1751+len(EFF_type),1),EFF_type[:,n],lw=2,label=EFF_lgd[n][22:])
    plt.legend(loc=0)
    plt.ylabel('fossil CO2 emissions (GtCO2/yr)')
    plt.xlabel('time (yr)')


# use the IRF to simulate past changes in atmospheric CO2
# conversion factor
k_CO2 = 0.1765 * 44 #{GtCO2/ppm}

# using the built-in convolution function
time = np.arange(len(EFF))
D_CO2_mod1 = 1/k_CO2 * np.convolve(EFF,irf_CO2(time))[:len(time)]

# explicitly calculating the convolution
D_CO2_mod2 = np.zeros(np.shape(EFF))
for t in range(len(EFF)):
    D_CO2_mod2[t] = 1/k_CO2 * np.sum(EFF[:t+1] * irf_CO2(np.arange(t+1))[::-1])

# using the corresponding box-model
D_CO2_mod3 = np.zeros(np.shape(EFF))
D_C0 = 0
D_C1 = 0
D_C2 = 0
D_C3 = 0
for t in range(len(EFF)):
    D_C0 += a_C0 * EFF[t]
    D_C1 += a_C1 * EFF[t] - D_C1/tau_C1
    D_C2 += a_C2 * EFF[t] - D_C2/tau_C2
    D_C3 += a_C3 * EFF[t] - D_C3/tau_C3
    D_CO2_mod3[t] = 1/k_CO2 * (D_C0+D_C1+D_C2+D_C3)


# load atmospheric CO2 reconstruction by IPCC
CO2_ipcc = np.array([line for line in csv.reader(open('data/HistAtmo_IPCC.1750-2011.CO2.csv','r'))],dtype=float)[:,0] #{ppm}

### Fig. 3.2 ###
# compare the three simulated atmospheric CO2 and the reconstruction
plt.figure()
plt.plot(np.arange(1751,1751+len(EFF)),D_CO2_mod1,label='built-in function')
plt.plot(np.arange(1751,1751+len(EFF)),D_CO2_mod2,label='hand-made convolution')
plt.plot(np.arange(1751,1751+len(EFF)),D_CO2_mod3,label='equivalent box-model')
plt.plot(np.arange(1750,1750+len(CO2_ipcc)),CO2_ipcc-CO2_ipcc[0],color='k',lw=2,ls='--',label='IPCC reconstruction')
plt.ylabel('excess in atmospheric CO2 (ppm)')
plt.xlabel('time (yr)')
plt.legend(loc=0)


#%%==========
# SECTION 4
#============

# inversion of ELUC needed to fit reconstructed CO2
ELUC_inv = np.zeros(np.shape(EFF))

# define error function
def err(e,t):
    # create ELUC vector used in error function
    ELUC_tmp = ELUC_inv.copy()
    ELUC_tmp[t:] = 0
    ELUC_tmp[t] = e
    # calculate simulated CO2    
    D_CO2_tmp =  1/k_CO2 * np.sum((EFF+ELUC_tmp)[:t+1] * irf_CO2(np.arange(t+1))[::-1])   
    return ( D_CO2_tmp - (CO2_ipcc[t+1]-CO2_ipcc[0]) )**2

# fit at every time-step
for t in range(len(EFF)):
    ELUC_inv[t] = fmin(err,[0],args=(t,),disp=False)

# load historical emissions from land-use change
ELUC_data = np.array([line for line in csv.reader(open('data/GCP+_ELUC.global.1850_2010.csv','r'))],dtype=float)[:,0] / 1000 * 44/12. #{GtCO2/yr}


### Fig. 4.1 ###
# compare ELUC from inversion and from data
time1=np.arange(1850,1850+len(ELUC_data))
plt.figure()
plt.plot(time1,ELUC_data)

### Fig. 4.2 ###
# compare ELUC from inversion and from data
plt.figure()
plt.plot(np.arange(1750,2012),np.zeros(2012-1750),'k-')
plt.plot(np.arange(1751,1751+len(ELUC_inv)),ELUC_inv,label='ELUC from inversion')
plt.plot(np.arange(1850,1850+len(ELUC_data)),ELUC_data,color='k',lw=2,label='ELUC from data')
plt.ylabel('CO2 emissions from land-use change (GtCO2/yr)')
plt.legend(loc=0)




